/* Pango
 * pango-layout-private.h: Private definitions for Pango Layout class
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __PANGO_LAYOUT_PRIVATE_H__
#define __PANGO_LAYOUT_PRIVATE_H__

#include <pango/pango-layout.h>

G_BEGIN_DECLS

#define LINE_IS_VALID(line) ((line)->layout != NULL)

typedef enum 
{
  BREAK_NONE_FIT,
  BREAK_SOME_FIT,
  BREAK_ALL_FIT,
  BREAK_EMPTY_FIT
} BreakResult;

typedef struct _ParaBreakState ParaBreakState;

struct _ParaBreakState
{
  GList *items;			/* This paragraph turned into items */
  gboolean first_line;		/* TRUE if this is the first line of the paragraph */
  int line_start_index;		/* Start index of line in layout->text */

  int remaining_width;		/* Amount of space remaining on line; < 0 is infinite */

  int start_offset;		/* Character offset of first item in state->items in layout->text */
  PangoGlyphString *glyphs;	/* Glyphs for the first item in state->items */
  PangoGlyphUnit *log_widths;	/* Logical widths for first item in state->items.. */
  int log_widths_offset;        /* Offset into log_widths to the point corresponding
				 * to the remaining portion of the first item */
};

struct _PangoLayout
{
  GObject parent_instance;

  /* If you add fields to PangoLayout, be sure to update both
   * the _copy function
   */
  
  PangoContext *context;
  PangoAttrList *attrs;
  PangoFontDescription *font_desc;
  
  gchar *text;
  int length;			/* length of text in bytes */
  int width;			/* wrap width, in device units */
  int indent;			/* amount by which first line should be shorter */
  int spacing;			/* spacing between lines */

  guint justify : 1;
  guint alignment : 2;

  guint single_paragraph : 1;
  
  gint n_chars;		        /* Total number of characters in layout */
  PangoLogAttr *log_attrs;	/* Logical attributes for layout's text */

  int tab_width;		/* Cached width of a tab. -1 == not yet calculated */

  PangoTabArray *tabs;
  
  GSList *lines;

  PangoWrapMode wrap;
};

struct _PangoLayoutClass
{
  GObjectClass parent_class;

  void         (*check_lines)             (PangoLayout *layout);
  void	       (*insert_run)		  (PangoLayoutLine *line,
					   ParaBreakState  *state,
					   const char      *text,
					   PangoItem       *run_item,
					   gboolean         last_run);
  void         (*set_font_description)    (PangoLayout                *layout,
					   const PangoFontDescription *desc);
  void         (*get_extents)             (PangoLayoutLine *line,
					   PangoRectangle  *ink_rect,
					   PangoRectangle  *logical_rect);
  PangoLayout* (*copy)                    (PangoLayout *src);
  void         (*copy_instance_variables) (PangoLayout *src,
					   PangoLayout *layout);
};

typedef struct _PangoLayoutRunPrivate PangoLayoutRunPrivate;
typedef struct _PangoLayoutRunPrivateClass PangoLayoutRunPrivateClass;

struct _PangoLayoutRunPrivate
{
  PangoLayoutRun run;
  PangoLayoutLine *line;

  PangoRectangle *ink_rect;
  PangoRectangle *logical_rect;

  gint baseline_adjust;

  /* Crammed into here because Pango doesn't have a PangoLayoutRun class */
  /*void (*get_extents) (PangoLayoutRun *run,
			   gboolean       *shape_setp,
			   PangoRectangle *run_ink,
			   PangoRectangle *run_logical);*/
};

typedef struct _PangoLayoutLinePrivate PangoLayoutLinePrivate;

struct _PangoLayoutLinePrivate
{
  PangoLayoutLine line;
  guint ref_count;
  PangoRectangle *ink_rect;
  PangoRectangle *logical_rect;
};


PangoLayout *pango_layout_new_real (PangoContext *context);
void         pango_layout_check_lines_real (PangoLayout *layout);
void     pango_layout_line_get_extents_real  (PangoLayoutLine  *line,
					      PangoRectangle   *ink_rect,
					      PangoRectangle   *logical_rect);
void pango_layout_line_get_empty_extents (PangoLayoutLine *line,
					  PangoRectangle  *logical_rect);
void pango_layout_run_get_extents (PangoLayoutRun *run,
				   gboolean       *shape_setp,
				   PangoRectangle *run_ink,
				   PangoRectangle *run_logical);
void pango_layout_run_get_extents_real (PangoLayoutRun *run,
					gboolean       *shape_setp,
					PangoRectangle *run_ink,
					PangoRectangle *run_logical);
void pango_layout_set_font_description_real (PangoLayout               *layout,
					     const PangoFontDescription *desc);

void imposed_extents (gint              n_chars,
		      PangoRectangle   *shape_ink,
		      PangoRectangle   *shape_logical,
		      PangoRectangle   *ink_rect,
		      PangoRectangle   *logical_rect);

void pango_layout_get_item_properties (PangoItem      *item,
				       PangoUnderline *uline,
				       gint           *rise,
				       PangoRectangle *ink_rect,
				       PangoRectangle *logical_rect,
				       gboolean       *shape_set);
PangoAttrList *pango_layout_get_effective_attributes (PangoLayout *layout);
void get_items_log_attrs (const char   *text,
			  GList        *items,
			  PangoLogAttr *log_attrs,
			  int           para_delimiter_len);
PangoLayoutLine * pango_layout_line_new         (PangoLayout     *layout);
gboolean can_break_at (PangoLayout *layout,
		       gint         offset,
		       gboolean     always_wrap_char);
gboolean can_break_in (PangoLayout *layout,
		       int          start_offset,
		       int          num_chars,
		       gboolean     allow_break_at_start);
PangoItem *uninsert_run (PangoLayoutLine *line);
void pango_layout_line_postprocess (PangoLayoutLine *line);
void insert_run (PangoLayoutLine *line,
		 ParaBreakState  *state,
		 const char      *text,
		 PangoItem       *run_item,
		 gboolean         last_run);
void imposed_shape (const char       *text,
		    gint              n_chars,
		    PangoRectangle   *shape_ink,
		    PangoRectangle   *shape_logical,
		    PangoGlyphString *glyphs);
void shape_tab (PangoLayoutLine  *line,
		PangoGlyphString *glyphs);



G_END_DECLS

#endif /* __PANGO_LAYOUT_PRIVATE_H__ */
